# -*- python -*-
# $RCSfile: rationalshort.py,v $
# $Revision: 1.10 $
# $Author: langer $
# $Date: 2008-02-07 19:24:11 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from ooflib.common import registeredclass
from ooflib.common.IO import parameter
from ooflib.common.IO import xmlmenudump
from ooflib.engine import rationalize
from ooflib.engine import rationalizefunctions

class RemoveShortSide(rationalize.Rationalizer):
    #
    #       C  _________
    #         |        / B   If a segment C-D is too short,
    #       D  \      /      it's better to merge C&D to form a triangle.
    #           \    /
    #            \  /
    #             \/      ===>     C ________
    #             A                  \      / B
    #                                 \    /
    #                                  \  /
    #                                   \/
    #                                    A
    #

    def __init__(self, ratio):
        self.ratio = ratio

    def findAndFix(self, skel, element):
        if element.nnodes() == 4:       # only applies to quads
            edgeLengths = element.getEdgeLengthsList()
            # See if the shortest edge is much shorter than the rest
            edgeLengths.sort()
            if edgeLengths[1] >= self.ratio*edgeLengths[0]:
                which = element.getShortestEdge()
                # Returns a list of ProvisionalChange objects
                return self.fix(skel, element, which)
        return []

    def fixAll(self, skel, element):
        if element.nnodes() == 4:
            which = element.getShortestEdge()
            return self.fix(skel, element, which)
        return []

    def fix(self, skel, element, which):
        return rationalizefunctions.removeShortSide(skel, element, which)

registeredclass.Registration(
    'Remove Short Sides',
    rationalize.Rationalizer,
    RemoveShortSide,
    gerund = 'removing short sides',
    ordering=0,
    params=[
    parameter.FloatParameter('ratio', value = 5.0,
                             tip = 'Maximum acceptable ratio of the lengths of the second shortest and the shortest sides.')
    ],
    tip = "Eliminate the shortest side of a quadrilateral.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/ration_short.xml'))
